"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.signRequest = signRequest;
exports.verifyEtag = verifyEtag;
exports.processApiResult = processApiResult;
exports.paginateApi = paginateApi;
exports.findIds = findIds;
exports.getEntityById = getEntityById;
exports.marvelAPI = void 0;

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const axios = require('axios');

const crypto = require('crypto');

const marvelAPI = 'https://gateway.marvel.com/v1/public/';
exports.marvelAPI = marvelAPI;

const paginateFilterCacheKey = (route, filter, offset, limit) => `${route}-${JSON.stringify(filter)}-${offset}-${limit}`;

const entityFilterCacheKey = (entity, id) => `${entity}-${id}`;

function signRequest({
  publicKey,
  privateKey
}) {
  const now = Date.now();
  return {
    hash: crypto.createHash('md5').update(`${now}${privateKey}${publicKey}`).digest('hex'),
    ts: now,
    apikey: publicKey
  };
}

function verifyEtag(_x) {
  return _verifyEtag.apply(this, arguments);
}

function _verifyEtag() {
  _verifyEtag = _asyncToGenerator(function* (_ref) {
    let cache = _ref.cache,
        route = _ref.route,
        cacheKey = _ref.cacheKey,
        options = _objectWithoutProperties(_ref, ["cache", "route", "cacheKey"]);

    const cached = yield cache.get(cacheKey);

    if (cached) {
      try {
        yield axios.get(route, {
          params: _objectSpread({}, signRequest(options)),
          headers: {
            Accept: '*/*',
            'If-None-Match': cached.etag
          },
          validateStatus: status => status === 304
        });
        return cached;
      } catch (err) {
        // ignore error
        return false;
      }
    }

    return false;
  });
  return _verifyEtag.apply(this, arguments);
}

function processApiResult(result) {
  return {
    data: result.data.data,
    etag: result.data.etag
  };
}

function queryWithOffset(_x2) {
  return _queryWithOffset.apply(this, arguments);
}

function _queryWithOffset() {
  _queryWithOffset = _asyncToGenerator(function* (_ref2) {
    let route = _ref2.route,
        params = _ref2.params,
        options = _objectWithoutProperties(_ref2, ["route", "params"]);

    const result = yield axios.get(route, {
      params: _objectSpread({}, params, signRequest(options))
    });
    return processApiResult(result);
  });
  return _queryWithOffset.apply(this, arguments);
}

function paginateApi(_x3) {
  return _paginateApi.apply(this, arguments);
}

function _paginateApi() {
  _paginateApi = _asyncToGenerator(function* (_ref3) {
    let route = _ref3.route,
        limit = _ref3.limit,
        params = _ref3.params,
        cache = _ref3.cache,
        options = _objectWithoutProperties(_ref3, ["route", "limit", "params", "cache"]);

    let result;
    let offset = 0;
    let page = 1;
    let results = []; // We query first without cache to get the updated total results.

    result = yield queryWithOffset(_objectSpread({
      route,
      params: _objectSpread({
        offset,
        limit
      }, params)
    }, options));
    results = results.concat(result.data.results);
    const total = result.data.total;

    while (results.length < total) {
      page += 1;
      offset = limit * (page - 1); // apparently marvel decided that etag is different for each paginated result

      const cacheKey = paginateFilterCacheKey(route, params, offset, limit);
      const cached = yield verifyEtag(_objectSpread({
        cache,
        route,
        cacheKey
      }, options));

      if (cached) {
        results = results.concat(cached.result.results);
      } else {
        result = yield queryWithOffset(_objectSpread({
          route,
          params: _objectSpread({
            offset,
            limit
          }, params)
        }, options));
        results = results.concat(result.data.results);
        yield cache.set(cacheKey, {
          results: result.data,
          etag: result.etag
        });
      }
    }

    return results;
  });
  return _paginateApi.apply(this, arguments);
}

function findIds(_x4) {
  return _findIds.apply(this, arguments);
}

function _findIds() {
  _findIds = _asyncToGenerator(function* (_ref4) {
    let entity = _ref4.entity,
        entityFilter = _ref4.entityFilter,
        cache = _ref4.cache,
        options = _objectWithoutProperties(_ref4, ["entity", "entityFilter", "cache"]);

    const results = yield paginateApi(_objectSpread({
      route: `${marvelAPI}${entity}`,
      params: _objectSpread({}, entityFilter),
      cache
    }, options));
    return results.map(result => result.id);
  });
  return _findIds.apply(this, arguments);
}

function getEntityById(_x5) {
  return _getEntityById.apply(this, arguments);
}

function _getEntityById() {
  _getEntityById = _asyncToGenerator(function* (_ref5) {
    let entity = _ref5.entity,
        id = _ref5.id,
        cache = _ref5.cache,
        options = _objectWithoutProperties(_ref5, ["entity", "id", "cache"]);

    const route = `${marvelAPI}${entity}/${id}`;
    const cacheKey = entityFilterCacheKey(entity, id);
    const cached = yield verifyEtag(_objectSpread({
      cache,
      route,
      cacheKey
    }, options));

    if (cached) {
      return {
        id,
        entityResult: cached.result.results[0]
      };
    }

    const result = processApiResult((yield axios.get(route, {
      params: _objectSpread({}, signRequest(options))
    })));
    yield cache.set(cacheKey, {
      result: result.data,
      etag: result.etag
    });
    return {
      id,
      entityResult: result.data.results[0]
    };
  });
  return _getEntityById.apply(this, arguments);
}