"use strict";

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const _ = require('lodash');

const crypto = require('crypto');

const _require = require('./api-helper'),
      marvelAPI = _require.marvelAPI,
      paginateApi = _require.paginateApi,
      findIds = _require.findIds,
      getEntityById = _require.getEntityById;

const defaultOptions = {
  limit: 100
};

function getEntitiesByFilter(_x) {
  return _getEntitiesByFilter.apply(this, arguments);
}

function _getEntitiesByFilter() {
  _getEntitiesByFilter = _asyncToGenerator(function* (_ref2) {
    let cache = _ref2.cache,
        entity = _ref2.entity,
        entityFilter = _ref2.entityFilter,
        options = _objectWithoutProperties(_ref2, ["cache", "entity", "entityFilter"]);

    const ids = yield findIds(_objectSpread({
      entity,
      cache,
      entityFilter
    }, options));
    return yield Promise.all(ids.map(id => getEntityById(_objectSpread({
      entity,
      id,
      cache
    }, options))));
  });
  return _getEntitiesByFilter.apply(this, arguments);
}

function getEntityResourcesById(_x2) {
  return _getEntityResourcesById.apply(this, arguments);
}

function _getEntityResourcesById() {
  _getEntityResourcesById = _asyncToGenerator(function* (_ref3) {
    let entity = _ref3.entity,
        id = _ref3.id,
        resource = _ref3.resource,
        cache = _ref3.cache,
        limit = _ref3.limit,
        options = _objectWithoutProperties(_ref3, ["entity", "id", "resource", "cache", "limit"]);

    return yield paginateApi(_objectSpread({
      route: `${marvelAPI}${entity}/${id}/${resource}`,
      limit,
      cache
    }, options));
  });
  return _getEntityResourcesById.apply(this, arguments);
}

function processEntities({
  actions: {
    createNode
  },
  entities
}) {
  _.keys(entities).forEach(entity => _.values(entities[entity]).forEach(child => {
    createNode(_objectSpread({
      parent: '__SOURCE__',
      internal: {
        type: `${entity}Node`,
        contentDigest: crypto.createHash('md5').update(JSON.stringify(child)).digest('hex')
      },
      children: []
    }, child, {
      id: `${child.id}`
    }));
  }));
}

function runQueries(_x3) {
  return _runQueries.apply(this, arguments);
}

function _runQueries() {
  _runQueries = _asyncToGenerator(function* (_ref4) {
    let cache = _ref4.cache,
        queries = _ref4.queries,
        options = _objectWithoutProperties(_ref4, ["cache", "queries"]);

    const entities = {
      characters: {},
      comics: {},
      series: {},
      events: {},
      creators: {}
    };

    const mergeResults = (entity, result) => {
      if (result.id in entities[entity]) {
        entities[entity][result.id] = _objectSpread({}, entities[entity][result.id], result);
      } else {
        entities[entity][result.id] = result;
      }
    };
    /* eslint-disable no-restricted-syntax */


    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = queries[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const _step$value = _step.value,
              entity = _step$value.entity,
              entityFilter = _step$value.entityFilter,
              resources = _step$value.resources;

        try {
          const filteredEntities = yield getEntitiesByFilter(_objectSpread({
            cache,
            entity,
            entityFilter
          }, options));
          var _iteratorNormalCompletion2 = true;
          var _didIteratorError2 = false;
          var _iteratorError2 = undefined;

          try {
            for (var _iterator2 = filteredEntities[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
              const filtered = _step2.value;

              if (!resources) {
                mergeResults(entity, filtered.entityResult);
              } else {
                var _iteratorNormalCompletion3 = true;
                var _didIteratorError3 = false;
                var _iteratorError3 = undefined;

                try {
                  for (var _iterator3 = resources[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                    const resource = _step3.value;
                    const resourceResult = yield getEntityResourcesById(_objectSpread({
                      entity,
                      resource,
                      cache,
                      id: filtered.id
                    }, options));
                    filtered.entityResult[`${resource}___NODE`] = resourceResult.map(res => `${res.id}`);
                    mergeResults(entity, filtered.entityResult);
                    resourceResult.forEach(res => mergeResults(resource, res));
                  }
                } catch (err) {
                  _didIteratorError3 = true;
                  _iteratorError3 = err;
                } finally {
                  try {
                    if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
                      _iterator3.return();
                    }
                  } finally {
                    if (_didIteratorError3) {
                      throw _iteratorError3;
                    }
                  }
                }
              }
            }
          } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
                _iterator2.return();
              }
            } finally {
              if (_didIteratorError2) {
                throw _iteratorError2;
              }
            }
          }
        } catch (err) {
          console.warn('Something went wrong querying Marvel', err); // eslint-disable-line no-console
        }
      }
      /* eslint-enable no-restricted-syntax */

    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return entities;
  });
  return _runQueries.apply(this, arguments);
}

exports.sourceNodes =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* ({
    actions,
    cache
  }, options) {
    const pluginOptions = _objectSpread({}, defaultOptions, options);

    const entities = yield runQueries(_objectSpread({
      cache,
      queries: pluginOptions.queries
    }, options));
    processEntities({
      actions,
      entities
    });
  });

  return function (_x4, _x5) {
    return _ref.apply(this, arguments);
  };
}();